/*
 * Copyright 1996 John D. Polstra.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by John Polstra.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *      $Id: rtld.h,v 1.1.1.1 1996/01/19 02:01:30 jdp Exp $
 */

#ifndef RTLD_H /* { */
#define RTLD_H 1

#include <stddef.h>
#include <sys/types.h>

#include "elf32.h"
#include "elf_386.h"

#define PAGESIZE	4096
#define round_down(x)	((x) & ~(PAGESIZE-1))
#define round_up(x)	round_down((x) + PAGESIZE - 1)

#define NEW(type)	((type *) xmalloc(sizeof(type)))
#define CNEW(type)	((type *) xcalloc(sizeof(type)))

typedef unsigned char bool;	/* Boolean */

typedef struct Struct_Needed_Entry {
    struct Struct_Needed_Entry *next;
    unsigned long name;		/* Offset of name in string table */
} Needed_Entry;

typedef struct Struct_Obj_Entry {
    struct Struct_Obj_Entry *next;
    char *path;
    int refcount;

    /* These items are computed by map_object() or by digest_phdr(). */
    caddr_t mapbase;		/* Base address of mapped region */
    size_t mapsize;		/* Size of mapped region in bytes */
    size_t textsize;		/* Size of text segment in bytes */
    Elf32_Addr vaddrbase;	/* Base address in shared object file */
    const Elf32_Dyn *dynamic;	/* Dynamic section */
    caddr_t entry;		/* Entry point */
    const Elf32_Phdr *phdr;	/* Program header if it is mapped, else NULL */
    size_t phsize;		/* Size of program header in bytes */

    /* Items from the dynamic section */
    Elf32_Addr *got;		/* GOT table */
    const Elf32_Rel *rel;	/* Relocation entries */
    unsigned long relsize;	/* Size in bytes of relocation info */
    const Elf32_Rel *pltrel;	/* PLT relocation entries */
    unsigned long pltrelsize;	/* Size in bytes of PLT relocation info */
    const Elf32_Sym *symtab;	/* Symbol table */
    const char *strtab;		/* String table */
    unsigned long strsize;	/* Size in bytes of string table */

    const Elf32_Word *buckets;	/* Hash table buckets array */
    unsigned long nbuckets;	/* Number of buckets */
    const Elf32_Word *chains;	/* Hash table chain array */
    unsigned long nchains;	/* Number of chains */

    Needed_Entry *needed;	/* Shared objects needed by this one */

    bool textrel;		/* True if there are relocations to text seg */
    bool mainprog;		/* True if this is the main program */
} Obj_Entry;

extern void error(const char *, ...);
extern Obj_Entry *map_object(int);
extern void *xcalloc(size_t);
extern void *xmalloc(size_t);
extern char *xstrdup(const char *);

#endif /* } */
